#include <Adafruit_GPS.h> //GPS
#include <SoftwareSerial.h> //GPS  -> ~300 bytes Storage and ~70 bytes global variables
#include <SPI.h> //SD  
#include <SD.h> //SD-> ~5250 bytes Storage and ~800 bytes global variables
#include <Wire.h> //BNO
#include <Adafruit_Sensor.h> //BNO
#include <Adafruit_BNO055.h> //BNO
#include <utility/imumaths.h> //BNO  -> ~1000 bytes of storage ~200 bytes of global variables
#include "IntersemaBaro.h"


//declare instances of classes
//note, I may have mixed up the two sesnors here, so simply switch true and false if the results don't really make sense.
Intersema::BaroPressure_MS5607B parallaxIn(true);//corresponds to CSB pin as ground
Intersema::BaroPressure_MS5607B parallaxOut(false);//corresponds to CSB pin as 3V

/***************************************************************/

// Connect the GPS Power pin to 5V
// Connect the GPS Ground pin to ground
//   Connect the GPS TX (transmit) pin to Arduino RX1, RX2 or RX3
//   Connect the GPS RX (receive) pin to matching TX1, TX2 or TX3

//BNO just works

/***************************************************************/

int32_t presIn=0;//Pascals
int32_t presOut=0;

int32_t tempIn=0;//Celsius*100
int32_t tempOut=0;

//alitutde based on pressure and comparison chart
int32_t altByPres=0;//Meters

//raw data collected by MS5607 sensors
int32_t rawD1In=0;
int32_t rawD2In=0;
int32_t rawD1Out=0;
int32_t rawD2Out=0;

const int valvePin1 = 3;
const int valvePin2 = 4;
int valveState1 = LOW;
int valveState2 = HIGH;

int temperatureBNO;
double qW;//Quaternion W
double qX;//Quaternion X
double qY;//Quaternion Y
double qZ;//Quaternion Z

double eulerX;//euler X - degrees
double eulerY;//euler Y - degrees
double eulerZ;//euler Z - degrees

double orientX;
double orientY;
double orientZ;

double accelX;//acceleration X - m/s^2
double accelY;//acceleration Y - m/s^2
double accelZ;//acceleration Z - m/s^2

double gyroX;// gyroscope X - rad/s
double gyroY;// gyroscope Y - rad/s
double gyroZ;// gyroscope Z - rad/s

double linAccelX;//linear acceleration X - m/s^2
double linAccelY;//linear acceleration Y - m/s^2
double linAccelZ;//linear acceleration Z - m/s^2

double gravityX;//gravity X - m/s^2
double gravityY;//gravity Y - m/s^2
double gravityZ;//gravity Z - m/s^2

double hours;
double minutes;
double seconds;
double milliseconds;

double day;
double month;
double year;

int fix;
int fixQ;//fix quality

double latitude;
char lat;
double longitude;
char lon;
double latitudeDegrees;//Works with google maps
double longitudeDegrees;//Works with google maps

double speedGPS;
double angle;
double altitudeGPS;//Altitude fro GPS in meters
int satillites;//Number of satillites

/***************************************************************/

HardwareSerial mySerial = Serial1;
Adafruit_GPS GPS(&Serial1);
Adafruit_BNO055 bno = Adafruit_BNO055(55);

#define GPSECHO  false

boolean usingInterrupt = false;
void useInterrupt(boolean);

/***************************************************************/

void setup() 
{

  parallaxIn.init();
  parallaxOut.init();
  
  pinMode(valvePin1, OUTPUT);
  pinMode(valvePin2, OUTPUT);

  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  Serial.print("\n\nInitializing SD card...");

  // see if the card is present and can be initialized:
  if (!SD.begin()) {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");
  
  
  GPS.begin(9600);
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA);
  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);
  GPS.sendCommand(PGCMD_ANTENNA);
  useInterrupt(true);
  delay(1000);

  bno.begin();
  bno.setExtCrystalUse(true);
  closeValve();
  delay(5000);
  noValve();
  int32_t PROM[6] = {0,0,0,0,0,0};
  parallaxIn.GetPromValues(PROM);
  String promStrIn = "PROM values for pressure sensor inside balloon are: " + (String)PROM[0] + ", " + (String)PROM[1] + ", " + (String)PROM[2] + ", " + (String)PROM[3] + ", " + (String)PROM[4] + ", " + (String)PROM[5];
  parallaxOut.GetPromValues(PROM);
  String promStrOut = "PROM values for pressure sensor outside balloon are: " + (String)PROM[0] + ", " + (String)PROM[1] + ", " + (String)PROM[2] + ", " + (String)PROM[3] + ", " + (String)PROM[4] + ", " + (String)PROM[5];
  File dataFile = SD.open("datalog.txt", FILE_WRITE);
  // if the file is available, write to it:
  if (dataFile) {
    dataFile.println(promStrIn);
    dataFile.println(promStrOut);
    dataFile.close();
    // print to the serial port too:
    Serial.println(promStrIn);
    Serial.println(promStrOut);
  }
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening datalog.txt");
  }
}

/***************************************************************/

//GPS STUFF
// Interrupt is called once a millisecond, looks for any new GPS data, and stores it
SIGNAL(TIMER0_COMPA_vect) {
  char c = GPS.read();
  // if you want to debug, this is a good time to do it!
#ifdef UDR0
  if (GPSECHO)
    if (c) UDR0 = c;  
    // writing direct to UDR0 is much much faster than Serial.print 
    // but only one character can be written at a time. 
#endif
}

void useInterrupt(boolean v) {
  if (v) {
    // Timer0 is already used for millis() - we'll just interrupt somewhere
    // in the middle and call the "Compare A" function above
    OCR0A = 0xAF;
    TIMSK0 |= _BV(OCIE0A);
    usingInterrupt = true;
  } else {
    // do not call the interrupt function COMPA anymore
    TIMSK0 &= ~_BV(OCIE0A);
    usingInterrupt = false;
  }
}

uint32_t timer = millis();

/***************************************************************/
/***************************************************************/
/***************************************************************/

void loop() 
{
  /*openValve();
  delay(5000);
  noValve();
  delay(5000);
  closeValve();
  delay(5000);
  noValve();
  delay(5000);*/
  int timerF = millis();
  delay(1000);
  writeBNO();
  writeGPS();
  setInternalPresTemp();
  setExternalPresTemp();
  writeLog(timerF);
}


/***************************************************************/
/***************************************************************/
/***************************************************************/

void writeBNO()
{
  int8_t temp = bno.getTemp();
  imu::Quaternion quat = bno.getQuat();
  imu::Vector<3> euler = bno.getVector(Adafruit_BNO055::VECTOR_EULER);
  imu::Vector<3> accelerometer = bno.getVector(Adafruit_BNO055::VECTOR_ACCELEROMETER);
  imu::Vector<3> gyroscope = bno.getVector(Adafruit_BNO055::VECTOR_GYROSCOPE);
  imu::Vector<3> linearaccel = bno.getVector(Adafruit_BNO055::VECTOR_LINEARACCEL);
  imu::Vector<3> gravity = bno.getVector(Adafruit_BNO055::VECTOR_GRAVITY);
  sensors_event_t event; 
  bno.getEvent(&event);

  int temperatureBNO;
  qW = quat.w();//Quaternion W
  qX = quat.x();//Quaternion X
  qY = quat.y();//Quaternion Y
  qZ = quat.z();//Quaternion Z
  eulerX = euler.x();//euler X - degrees
  eulerY = euler.y();//euler Y - degrees
  eulerZ = euler.z();//euler Z - degrees
  orientX = event.orientation.x;
  orientY = event.orientation.y;
  orientZ = event.orientation.z;
  accelX = accelerometer.x();//acceleration X - m/s^2
  accelY = accelerometer.y();//acceleration Y - m/s^2
  accelZ = accelerometer.z();//acceleration Z - m/s^2
  gyroX = gyroscope.x();// gyroscope X - rad/s
  gyroY = gyroscope.y();// gyroscope Y - rad/s
  gyroZ = gyroscope.z();// gyroscope Z - rad/s
  linAccelX = linearaccel.x();//linear acceleration X - m/s^2
  linAccelY = linearaccel.y();//linear acceleration Y - m/s^2
  linAccelZ = linearaccel.z();//linear acceleration Z - m/s^2
  gravityX = gravity.x();//gravity X - m/s^2
  gravityY = gravity.y();//gravity Y - m/s^2
  gravityZ = gravity.z();//gravity Z - m/s^2
}

/***************************************************************/

void writeGPS()
{
  if (GPS.newNMEAreceived()) {
    // a tricky thing here is if we print the NMEA sentence, or data
    // we end up not listening and catching other sentences! 
    // so be very wary if using OUTPUT_ALLDATA and trying to print out data
    //Serial.println(GPS.lastNMEA());   // this also sets the newNMEAreceived() flag to false
  
    if (!GPS.parse(GPS.lastNMEA())) {}  // this also sets the newNMEAreceived() flag to false
     // out = "fail in parse";  // a**originally return; we can fail to parse a sentence in which case we should just wait for another
  }
  // if millis() or timer wraps around, we'll just reset it
  if (timer > millis())  timer = millis();

  // approximately every 2 seconds or so, print out the current stats
  //if (millis() - timer > 2000 ) { 
    timer = millis(); // reset the timer

  hours = GPS.hour;
  minutes = GPS.minute;
  seconds = GPS.seconds;
  milliseconds = GPS.milliseconds;
  day = GPS.day;
  month = GPS.month;
  year = GPS.year;
  fix = (int)GPS.fix;
  fixQ = (int)GPS.fixquality;
  latitude = GPS.latitude;
  lat = GPS.lat;
  longitude = GPS.longitude;
  lon = GPS.lon;
  latitudeDegrees = GPS.latitudeDegrees;//Works with google maps
  longitudeDegrees = GPS.longitudeDegrees;//Works with google maps
  speedGPS = GPS.speed;
  angle = GPS.angle;
  altitudeGPS = GPS.altitude;//Altitude fro GPS in meters
  satillites = (int)GPS.satellites;//Number of satillites  
  //}
}

void setInternalPresTemp(){//updates variables related to sensor inside balloon wheel
  //currently only takes sample once, if we want averages, tell me
  int32_t output[4] = {0,0,0,0};
  parallaxIn.SetTemperaturePressure(output);
  presIn = output[0];
  tempIn = output[1];
  rawD1In = output[2];
  rawD2In = output[3];
}

void setExternalPresTemp(){//updates variables related to the sensor outside balloon
  int32_t output[4] = {0,0,0,0};
  parallaxOut.SetTemperaturePressure(output);
  presOut = output[0];
  tempOut = output[1];
  rawD1Out = output[2];
  rawD2Out = output[3];
}
void setAltByPres(){//uses chart to approximate height
  altByPres = parallaxOut.PascalToCentimeter(presOut)/100;
}

void openValve(){
  valveState1 = HIGH;
  valveState2 = LOW;
  digitalWrite(valvePin1, valveState1);
  digitalWrite(valvePin2, valveState2);
}

void closeValve(){
  valveState1 = LOW;
  valveState2 = HIGH;
  digitalWrite(valvePin2, valveState2);
  digitalWrite(valvePin1, valveState1);
}

void noValve(){
  valveState1 = LOW;
  valveState2 = LOW;
  digitalWrite(valvePin1, valveState1);
  digitalWrite(valvePin2, valveState2);
}

void writeLog(int timerF){
  // make a string for assembling the data to log:
  String dataString = "";
  dataString += "Time: " + (String)timer;
  dataString += "\tpresIn: " + (String)presIn;
  dataString += "\tpresOut: " + (String)presOut;
  dataString += "\ttempIn: " + (String)tempIn;
  dataString += "\ttempOut: " + (String)tempOut;
  dataString += "\trawD1In: " + (String)rawD1In;
  dataString += "\trawD2In: " + (String)rawD2In;
  dataString += "\trawD1Out: " + (String)rawD1Out;
  dataString += "\trawD2Out: " + (String)rawD2Out + "\t|";
  dataString += "\tvalveS1: " + (String)valveState1;
  dataString += "\tvalveS2: " + (String)valveState2 + "\t|";
  dataString += "\ttempBNO: " + (String)temperatureBNO;
  dataString += "\tqW:" + (String)qW;
  dataString += "\tqX:" + (String)qX;
  dataString += "\tqY:" + (String)qY;
  dataString += "\tqZ:" + (String)qZ;
  dataString += "\teulerX: " + (String)eulerX;
  dataString += "\teulerY: " + (String)eulerY;
  dataString += "\t  eulerZ: " + (String)eulerZ;
  dataString += "\torientX: " + (String)orientX;
  dataString += "\torientY: " + (String)orientY;
  dataString += "\torientZ: " + (String)orientZ;
  dataString += "\taccelX: " + (String)accelX;
  dataString += "\taccelY: " + (String)accelY;
  dataString += "\taccelZ: " + (String)accelZ;
  dataString += "\tgyroX: " + (String)gyroX;
  dataString += "\tgyroY: " + (String)gyroY;
  dataString += "\tgyroZ: " + (String)gyroZ;
  dataString += "  \tlinAccelX: " + (String)linAccelX;
  dataString += "  \tlinAccelY: " + (String)linAccelY;
  dataString += "  \tlinAccelZ: " + (String)linAccelZ;
  dataString += "  \tgravityX: " + (String)gravityX;
  dataString += "\tgravityY: " + (String)gravityY;
  dataString += "\tgravityZ: " + (String)gravityZ + "\t|";
  dataString += "\thours: " + (String)hours;
  dataString += "\tminutes: " + (String)minutes;
  dataString += "\tseconds: " + (String)seconds;
  dataString += "\tmilliseconds: " + (String)milliseconds;
  dataString += "\tfix: " + (String)fix;
  dataString += "\tfixQ: " + (String)fixQ;
  dataString += "\tlatitude: " + (String)latitude;
  dataString += "\tlat: " + (String)lat;
  dataString += "\tlongitude: " + (String)longitude;
  dataString += "\tlon: " + (String)lon;
  dataString += "\tlatDegrees: " + (String)latitudeDegrees;
  dataString += "\tlongDegrees: " + (String)longitudeDegrees;
  dataString += "\tspeedGPS: " + (String)speedGPS;
  dataString += "\tangle: " + (String)angle;
  dataString += "\taltitudeGPS: " + (String)altitudeGPS;
  dataString += "\tdelay: " + (String)(millis()-timerF);


  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  File dataFile = SD.open("datalog.txt", FILE_WRITE);

  // if the file is available, write to it:
  if (dataFile) {
    dataFile.println(dataString);
    dataFile.close();
    // print to the serial port too:
    Serial.println(dataString);
  }
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening datalog.txt");
  }
}



