#include <Adafruit_GPS.h> //GPS
#include <SoftwareSerial.h> //GPS  -> ~300 bytes Storage and ~70 bytes global variables
#include <SPI.h> //SD  
#include <SD.h> //SD-> ~5250 bytes Storage and ~800 bytes global variables
#include <Wire.h> //BNO
#include <Adafruit_Sensor.h> //BNO
#include <Adafruit_BNO055.h> //BNO
#include <utility/imumaths.h> //BNO  -> ~1000 bytes of storage ~200 bytes of global variables

/***************************************************************/

// Connect the GPS Power pin to 5V
// Connect the GPS Ground pin to ground
//   Connect the GPS TX (transmit) pin to Arduino RX1, RX2 or RX3
//   Connect the GPS RX (receive) pin to matching TX1, TX2 or TX3

//BNO just works

/***************************************************************/

// - VECTOR_ACCELEROMETER - m/s^2
  // - VECTOR_MAGNETOMETER  - uT
  // - VECTOR_GYROSCOPE     - rad/s
  // - VECTOR_EULER         - degrees
  // - VECTOR_LINEARACCEL   - m/s^2
  // - VECTOR_GRAVITY       - m/s^2

int temperatureBNO;
double qW;//Quaternion W
double qX;//Quaternion X
double qY;//Quaternion Y
double qZ;//Quaternion Z
double eulerX;//euler X - degrees
double eulerY;//euler Y - degrees
double eulerZ;//euler Z - degrees
double orientX;
double orientY;
double orientZ;
double accelX;//acceleration X - m/s^2
double accelY;//acceleration Y - m/s^2
double accelZ;//acceleration Z - m/s^2
double gyroX;// gyroscope X - rad/s
double gyroY;// gyroscope Y - rad/s
double gyroZ;// gyroscope Z - rad/s
double linAccelX;//linear acceleration X - m/s^2
double linAccelY;//linear acceleration Y - m/s^2
double linAccelZ;//linear acceleration Z - m/s^2
double gravityX;//gravity X - m/s^2
double gravityY;//gravity Y - m/s^2
double gravityZ;//gravity Z - m/s^2
double hours;
double minutes;
double seconds;
double milliseconds;
double day;
double month;
double year;
int fix;
int fixQ;//fix quality
double latitude;
char lat;
double longitude;
char lon;
double latitudeDegrees;//Works with google maps
double longitudeDegrees;//Works with google maps
double speedGPS;
double angle;
double altitudeGPS;//Altitude fro GPS in meters
int satillites;//Number of satillites

/***************************************************************/

HardwareSerial mySerial = Serial1;
Adafruit_GPS GPS(&Serial1);
Adafruit_BNO055 bno = Adafruit_BNO055(55);

#define GPSECHO  false

boolean usingInterrupt = false;
void useInterrupt(boolean);

/***************************************************************/

void setup() 
{
  Serial.begin(115000);
  GPS.begin(9600);
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA);
  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);
  GPS.sendCommand(PGCMD_ANTENNA);
  useInterrupt(true);
  delay(1000);

  bno.begin();
  bno.setExtCrystalUse(true);
}

/***************************************************************/

//GPS STUFF
// Interrupt is called once a millisecond, looks for any new GPS data, and stores it
SIGNAL(TIMER0_COMPA_vect) {
  char c = GPS.read();
  // if you want to debug, this is a good time to do it!
#ifdef UDR0
  if (GPSECHO)
    if (c) UDR0 = c;  
    // writing direct to UDR0 is much much faster than Serial.print 
    // but only one character can be written at a time. 
#endif
}

void useInterrupt(boolean v) {
  if (v) {
    // Timer0 is already used for millis() - we'll just interrupt somewhere
    // in the middle and call the "Compare A" function above
    OCR0A = 0xAF;
    TIMSK0 |= _BV(OCIE0A);
    usingInterrupt = true;
  } else {
    // do not call the interrupt function COMPA anymore
    TIMSK0 &= ~_BV(OCIE0A);
    usingInterrupt = false;
  }
}

uint32_t timer = millis();

/***************************************************************/

void loop() 
{
  int timerF = millis();
  writeBNO();
  writeGPS();
  Serial.print(qW); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(qX); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(qY); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(qZ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(eulerX); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(eulerY); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(eulerZ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(orientX); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(orientY); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(orientZ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(accelX); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(accelY); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(accelZ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(gyroX); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(gyroY); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(gyroZ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(linAccelX); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(linAccelY); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(linAccelZ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(gravityX); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(gravityY); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(gravityZ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(hours); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(minutes); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(seconds); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(milliseconds); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(day); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(month); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(year); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(fix); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(fixQ); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(latitude); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(lat); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(longitude); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(lon); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(latitudeDegrees); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(longitudeDegrees); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(speedGPS); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(angle); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(altitudeGPS); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(satillites); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.print(millis()-timerF); Serial.print("\t");Serial.print("|");Serial.print("\t");
  Serial.println("");
}

/***************************************************************/
/***************************************************************/

void writeBNO()
{
  int8_t temp = bno.getTemp();
  imu::Quaternion quat = bno.getQuat();
  imu::Vector<3> euler = bno.getVector(Adafruit_BNO055::VECTOR_EULER);
  imu::Vector<3> accelerometer = bno.getVector(Adafruit_BNO055::VECTOR_ACCELEROMETER);
  imu::Vector<3> gyroscope = bno.getVector(Adafruit_BNO055::VECTOR_GYROSCOPE);
  imu::Vector<3> linearaccel = bno.getVector(Adafruit_BNO055::VECTOR_LINEARACCEL);
  imu::Vector<3> gravity = bno.getVector(Adafruit_BNO055::VECTOR_GRAVITY);
  sensors_event_t event; 
  bno.getEvent(&event);

  int temperatureBNO;
  qW = quat.w();//Quaternion W
  qX = quat.x();//Quaternion X
  qY = quat.y();//Quaternion Y
  qZ = quat.z();//Quaternion Z
  eulerX = euler.x();//euler X - degrees
  eulerY = euler.y();//euler Y - degrees
  eulerZ = euler.z();//euler Z - degrees
  orientX = event.orientation.x;
  orientY = event.orientation.y;
  orientZ = event.orientation.z;
  accelX = accelerometer.x();//acceleration X - m/s^2
  accelY = accelerometer.y();//acceleration Y - m/s^2
  accelZ = accelerometer.z();//acceleration Z - m/s^2
  gyroX = gyroscope.x();// gyroscope X - rad/s
  gyroY = gyroscope.y();// gyroscope Y - rad/s
  gyroZ = gyroscope.z();// gyroscope Z - rad/s
  linAccelX = linearaccel.x();//linear acceleration X - m/s^2
  linAccelY = linearaccel.y();//linear acceleration Y - m/s^2
  linAccelZ = linearaccel.z();//linear acceleration Z - m/s^2
  gravityX = gravity.x();//gravity X - m/s^2
  gravityY = gravity.y();//gravity Y - m/s^2
  gravityZ = gravity.z();//gravity Z - m/s^2
}

/***************************************************************/

void writeGPS()
{
  if (GPS.newNMEAreceived()) {
    // a tricky thing here is if we print the NMEA sentence, or data
    // we end up not listening and catching other sentences! 
    // so be very wary if using OUTPUT_ALLDATA and trying to print out data
    //Serial.println(GPS.lastNMEA());   // this also sets the newNMEAreceived() flag to false
  
    if (!GPS.parse(GPS.lastNMEA())) {}  // this also sets the newNMEAreceived() flag to false
     // out = "fail in parse";  // a**originally return; we can fail to parse a sentence in which case we should just wait for another
  }
  // if millis() or timer wraps around, we'll just reset it
  if (timer > millis())  timer = millis();

  // approximately every 2 seconds or so, print out the current stats
  //if (millis() - timer > 2000 ) { 
    timer = millis(); // reset the timer

  hours = GPS.hour;
  minutes = GPS.minute;
  seconds = GPS.seconds;
  milliseconds = GPS.milliseconds;
  day = GPS.day;
  month = GPS.month;
  year = GPS.year;
  fix = (int)GPS.fix;
  fixQ = (int)GPS.fixquality;
  latitude = GPS.latitude;
  lat = GPS.lat;
  longitude = GPS.longitude;
  lon = GPS.lon;
  latitudeDegrees = GPS.latitudeDegrees;//Works with google maps
  longitudeDegrees = GPS.longitudeDegrees;//Works with google maps
  speedGPS = GPS.speed;
  angle = GPS.angle;
  altitudeGPS = GPS.altitude;//Altitude fro GPS in meters
  satillites = (int)GPS.satellites;//Number of satillites  
  //}
}


