/* Pin Notes:
 * VDD pin is connected to 3V with a 120nF or larger decoupling capactior between VDD and ground.
 * GND pni is connected to ground
 * CSB pin is ground for sensor inside balloon and 3V for sensor outside balloon
 * SCKL pin connected to SCL pin on arduino
 * SDO pin unused
 * SDI pin is connected to SDA pin on arduino (The circuit diagram also shows 3V and a 9400 Ohm resistor connected there as well. Is this necessary?)
 * PS pin is connected to 3V as well (definition of I2C communication)
*/
/* Personal Notes
 * The instructions emphasize keeping the two measurement ports in the center of the board completely clean and free from bright light,
 *  so we likely want to avoid putting the sensor on the side of the box.
 * The library included with this file is not the original! The original only has one function, which while cool is not what we need. I will try to make my edits obvious, but be aware of this change.
 */

//include libraries
#include <Wire.h>//Is this needed?
#include "IntersemaBaro.h"

//declare instances of classes
//note, I may have mixed up the two sesnors here, so simply switch true and false if the results don't really make sense.
Intersema::BaroPressure_MS5607B parallaxIn(true);//corresponds to CSB pin as ground
Intersema::BaroPressure_MS5607B parallaxOut(false);//corresponds to CSB pin as 3V

//global variables
int32_t presIn=0;//Pascals
int32_t presOut=0;
int32_t tempIn=0;//Kelvin
int32_t tempOut=0;
long time = 0;

//functions I've written
void setInternalPresTemp();
void setExternalPresTemp();


void setup() {
  parallaxIn.init();
  parallaxOut.init();
}

void loop() {
  //this code is to test how long reading takes
  time = millis();
  setInternalPresTemp();
  Serial.print("pressure: "); Serial.print(presIn, DEC);
  Serial.print(",  temperature: "); Serial.print(tempIn, DEC);
  Serial.print(",  delay: "); Serial.println(millis()-time, DEC);
}

void setInternalPresTemp(){//currently only takes sample once, if we want averages, tell me
  int32_t output[2] = {0,0};//we could set these to the current values, but ideally that will never matter
  parallaxIn.SetTemperaturePressure(output);
  presIn = output[0];
  tempIn = output[1];
}

void setExternalPresTemp(){
  int32_t output[2] = {0,0};
  parallaxOut.SetTemperaturePressure(output);
  presOut = output[0];
  tempOut = output[1];
}

