/* Pin Notes:
 * VDD pin is connected to 3V
 * GND pni is connected to ground
 * CSB pin is ground for sensor outside balloon and 3V for sensor inside balloon
 * SCKL pin connected to SCL pin on arduino
 * SDO pin unused
 * SDI pin is connected to SDA pin on arduino (The circuit diagram also shows 3V and a 9400 Ohm resistor connected there as well. Is this necessary?)
 * PS pin is connected to 3V as well (definition of I2C communication)
*/
/* Personal Notes
 * The instructions emphasize keeping the two measurement ports in the center of the board completely clean and free from bright light,
 *  so we likely want to avoid putting the sensor on the side of the box.
 * The library included with this file is not the original! The original only has one function, which while cool is not what we need. I will try to make my edits obvious, but be aware of this change.
 */

//include libraries
#include <Wire.h>//Is this needed?
#include "IntersemaBaro.h"

//declare instances of classes
//note, I may have mixed up the two sesnors here, so simply switch true and false if the results don't really make sense.
Intersema::BaroPressure_MS5607B parallaxIn(true);//corresponds to CSB pin as ground
Intersema::BaroPressure_MS5607B parallaxOut(false);//corresponds to CSB pin as 3V

//global variables
//temperature and pressure inside and outside balloon
int32_t presIn=0;//Pascals
int32_t presOut=0;
int32_t tempIn=0;//Celsius*100
int32_t tempOut=0;
//alitutde based on pressure and comparison chart
int32_t altByPres=0;//Meters
//raw data collected by MS5607 sensors
int32_t rawD1In=0;
int32_t rawD2In=0;
int32_t rawD1Out=0;
int32_t rawD2Out=0;
long timer = 0;

//functions I've written
void setInternalPresTemp();
void setExternalPresTemp();


void setup() {
  Serial.begin(9600);
  Serial.println("This is the data collected by two MS5607 sensors during the trip on the balloon");
  parallaxIn.init();
  parallaxOut.init();//Note, when the Serial says that "MS5607 Initiated", it does not actually mean it has been recognized, only that the code has started
  Serial.println("Pressure Inside (Pa), Temperature Inside (C*100), | Pressure Outside (Pa), Temperature Outside (C*100), | Aproximate Height (m), Delay (ms), | Raw Press In, Raw Temp In, | Raw Pres Out, Raw Temp In");
}

void loop() {
  //this code is to test how long reading takes
  delay(500);
  setInternalPresTemp();
  setExternalPresTemp();
  setAltByPres(); //no guarantee of accuracy
  Serial.print(presIn, DEC); Serial.print(",   ");
  Serial.print(tempIn, DEC); Serial.print(",   |   ");
  Serial.print(presOut, DEC); Serial.print(",   ");
  Serial.print(tempOut, DEC); Serial.print(",   |   ");
  Serial.print(altByPres); Serial.print(",   ");
  Serial.print(millis()-timer, DEC); timer = millis(); Serial.print(",   |   ");// it takes about 22 milliseconds to read temp and pres per sensor
  Serial.print(rawD1In, DEC); Serial.print(",   ");
  Serial.print(rawD2In, DEC); Serial.print(",   |   ");
  Serial.print(rawD1Out, DEC); Serial.print(",   ");
  Serial.print(rawD2Out, DEC); Serial.println(",--");//signals end of line
  //Need to adjust it to give raw output as well
}

void setInternalPresTemp(){//updates variables related to sensor inside balloon wheel
  //currently only takes sample once, if we want averages, tell me
  int32_t output[4] = {0,0,0,0};
  parallaxIn.SetTemperaturePressure(output);
  presIn = output[0];
  tempIn = output[1];
  rawD1In = output[2];
  rawD2In = output[3];
}

void setExternalPresTemp(){//updates variables related to the sensor outside balloon
  int32_t output[4] = {0,0,0,0};
  parallaxOut.SetTemperaturePressure(output);
  presOut = output[0];
  tempOut = output[1];
  rawD1Out = output[2];
  rawD2Out = output[3];
}
void setAltByPres(){//uses chart to approximate height
  altByPres = parallaxOut.PascalToCentimeter(presOut)/100;
}

